package betterquesting.questing;

import betterquesting.api.api.QuestingAPI;
import betterquesting.api.enums.EnumLogic;
import betterquesting.api.enums.EnumQuestState;
import betterquesting.api.enums.EnumQuestVisibility;
import betterquesting.api.properties.IPropertyType;
import betterquesting.api.properties.NativeProps;
import betterquesting.api.questing.IQuest;
import betterquesting.api.questing.rewards.IReward;
import betterquesting.api.questing.tasks.ITask;
import betterquesting.api.utils.BigItemStack;
import betterquesting.api.utils.NBTConverter;
import betterquesting.api2.cache.QuestCache;
import betterquesting.api2.storage.DBEntry;
import betterquesting.api2.storage.IDatabaseNBT;
import betterquesting.api2.utils.ParticipantInfo;
import betterquesting.core.BetterQuesting;
import betterquesting.questing.rewards.RewardStorage;
import betterquesting.questing.tasks.TaskStorage;
import betterquesting.storage.PropertyContainer;
import betterquesting.storage.QuestSettings;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Items;
import net.minecraft.nbt.NBTBase;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagIntArray;
import net.minecraft.nbt.NBTTagList;
import org.apache.logging.log4j.Level;

/* loaded from: betterquesting_questing.jar:betterquesting/questing/QuestInstance.class */
public class QuestInstance implements IQuest {
    private final TaskStorage tasks = new TaskStorage();
    private final RewardStorage rewards = new RewardStorage();
    private final HashMap<UUID, NBTTagCompound> completeUsers = new HashMap<>();
    private int[] preRequisites = new int[0];
    private final PropertyContainer qInfo = new PropertyContainer();

    public /* bridge */ /* synthetic */ NBTBase writeProgressToNBT(NBTBase nBTBase, @Nullable List list) {
        return writeProgressToNBT((NBTTagCompound) nBTBase, (List<UUID>) list);
    }

    public QuestInstance() {
        setupProps();
    }

    private void setupProps() {
        setupValue(NativeProps.NAME, "New Quest");
        setupValue(NativeProps.DESC, "No Description");
        setupValue(NativeProps.ICON, new BigItemStack(Items.field_151156_bN));
        setupValue(NativeProps.SOUND_COMPLETE);
        setupValue(NativeProps.SOUND_UPDATE);
        setupValue(NativeProps.LOGIC_QUEST, EnumLogic.AND);
        setupValue(NativeProps.LOGIC_TASK, EnumLogic.AND);
        setupValue(NativeProps.REPEAT_TIME, -1);
        setupValue(NativeProps.REPEAT_REL, true);
        setupValue(NativeProps.LOCKED_PROGRESS, false);
        setupValue(NativeProps.AUTO_CLAIM, false);
        setupValue(NativeProps.SILENT, false);
        setupValue(NativeProps.MAIN, false);
        setupValue(NativeProps.GLOBAL_SHARE, false);
        setupValue(NativeProps.SIMULTANEOUS, false);
        setupValue(NativeProps.VISIBILITY, EnumQuestVisibility.NORMAL);
    }

    /* JADX WARN: Multi-variable type inference failed */
    private <T> void setupValue(IPropertyType<T> prop) {
        setupValue(prop, prop.getDefault());
    }

    private <T> void setupValue(IPropertyType<T> prop, T def) {
        this.qInfo.setProperty(prop, this.qInfo.getProperty(prop, def));
    }

    public void update(EntityPlayer player) {
        UUID playerID = QuestingAPI.getQuestingUUID(player);
        int done = 0;
        for (DBEntry<ITask> entry : this.tasks.getEntries()) {
            if (((ITask) entry.getValue()).isComplete(playerID)) {
                done++;
            }
        }
        if (this.tasks.size() <= 0 || ((EnumLogic) this.qInfo.getProperty(NativeProps.LOGIC_TASK)).getResult(done, this.tasks.size())) {
            setComplete(playerID, System.currentTimeMillis());
        } else if (done > 0 && ((Boolean) this.qInfo.getProperty(NativeProps.SIMULTANEOUS)).booleanValue()) {
            resetUser(playerID, false);
        }
    }

    public void detect(EntityPlayer player) {
        UUID playerID = QuestingAPI.getQuestingUUID(player);
        QuestCache qc = player.getExtendedProperties(QuestCache.LOC_QUEST_CACHE.toString());
        if (qc == null) {
            return;
        }
        int questID = QuestDatabase.INSTANCE.getID(this);
        if ((isComplete(playerID) && (((Integer) this.qInfo.getProperty(NativeProps.REPEAT_TIME)).intValue() < 0 || this.rewards.size() <= 0)) || !canSubmit(player)) {
            return;
        }
        if (isUnlocked(playerID) || ((Boolean) QuestSettings.INSTANCE.getProperty(NativeProps.EDIT_MODE)).booleanValue()) {
            int done = 0;
            boolean update = false;
            ParticipantInfo partInfo = new ParticipantInfo(player);
            DBEntry<IQuest> dbe = new DBEntry<>(questID, this);
            for (DBEntry<ITask> entry : this.tasks.getEntries()) {
                if (!((ITask) entry.getValue()).isComplete(playerID)) {
                    ((ITask) entry.getValue()).detect(partInfo, dbe);
                    if (((ITask) entry.getValue()).isComplete(playerID)) {
                        done++;
                        update = true;
                    }
                } else {
                    done++;
                }
            }
            if (this.tasks.size() <= 0 || ((EnumLogic) this.qInfo.getProperty(NativeProps.LOGIC_TASK)).getResult(done, this.tasks.size())) {
                if (((Boolean) QuestSettings.INSTANCE.getProperty(NativeProps.EDIT_MODE)).booleanValue()) {
                    setComplete(playerID, System.currentTimeMillis());
                }
                qc.markQuestDirty(questID);
            } else if (update && ((Boolean) this.qInfo.getProperty(NativeProps.SIMULTANEOUS)).booleanValue()) {
                resetUser(playerID, false);
                qc.markQuestDirty(questID);
            } else if (update) {
                qc.markQuestDirty(questID);
            }
        }
    }

    public boolean hasClaimed(UUID uuid) {
        if (this.rewards.size() <= 0) {
            return true;
        }
        synchronized (this.completeUsers) {
            if (((Boolean) this.qInfo.getProperty(NativeProps.GLOBAL)).booleanValue() && !((Boolean) this.qInfo.getProperty(NativeProps.GLOBAL_SHARE)).booleanValue()) {
                Iterator<NBTTagCompound> it = this.completeUsers.values().iterator();
                while (it.hasNext()) {
                    if (it.next().func_74767_n("claimed")) {
                        return true;
                    }
                }
                return false;
            }
            NBTTagCompound entry = getCompletionInfo(uuid);
            return entry != null && entry.func_74767_n("claimed");
        }
    }

    public boolean canClaim(EntityPlayer player) {
        UUID pID = QuestingAPI.getQuestingUUID(player);
        NBTTagCompound entry = getCompletionInfo(pID);
        if (entry == null || hasClaimed(pID) || canSubmit(player)) {
            return false;
        }
        DBEntry<IQuest> dbe = new DBEntry<>(QuestDatabase.INSTANCE.getID(this), this);
        for (DBEntry<IReward> rew : this.rewards.getEntries()) {
            if (!((IReward) rew.getValue()).canClaim(player, dbe)) {
                return false;
            }
        }
        return true;
    }

    public void claimReward(EntityPlayer player) {
        int questID = QuestDatabase.INSTANCE.getID(this);
        DBEntry<IQuest> dbe = new DBEntry<>(questID, this);
        for (DBEntry<IReward> rew : this.rewards.getEntries()) {
            ((IReward) rew.getValue()).claimReward(player, dbe);
        }
        UUID pID = QuestingAPI.getQuestingUUID(player);
        QuestCache qc = player.getExtendedProperties(QuestCache.LOC_QUEST_CACHE.toString());
        synchronized (this.completeUsers) {
            NBTTagCompound entry = getCompletionInfo(pID);
            if (entry == null) {
                entry = new NBTTagCompound();
                this.completeUsers.put(pID, entry);
            }
            entry.func_74757_a("claimed", true);
            entry.func_74772_a("timestamp", System.currentTimeMillis());
        }
        if (qc != null) {
            qc.markQuestDirty(QuestDatabase.INSTANCE.getID(this));
        }
    }

    public boolean canSubmit(EntityPlayer player) {
        if (player == null) {
            return false;
        }
        UUID playerID = QuestingAPI.getQuestingUUID(player);
        synchronized (this.completeUsers) {
            NBTTagCompound entry = getCompletionInfo(playerID);
            if (entry == null) {
                return true;
            }
            if (!entry.func_74767_n("claimed") && ((Integer) getProperty(NativeProps.REPEAT_TIME)).intValue() >= 0) {
                if (this.tasks.size() <= 0) {
                    return true;
                }
                int done = 0;
                for (DBEntry<ITask> tsk : this.tasks.getEntries()) {
                    if (((ITask) tsk.getValue()).isComplete(playerID)) {
                        done++;
                    }
                }
                return !((EnumLogic) this.qInfo.getProperty(NativeProps.LOGIC_TASK)).getResult(done, this.tasks.size());
            }
            return false;
        }
    }

    public boolean isUnlocked(UUID uuid) {
        if (this.preRequisites.length <= 0) {
            return true;
        }
        int A = 0;
        int B = this.preRequisites.length;
        for (DBEntry<IQuest> quest : QuestDatabase.INSTANCE.bulkLookup(getRequirements())) {
            if (((IQuest) quest.getValue()).isComplete(uuid)) {
                A++;
            }
        }
        return ((EnumLogic) this.qInfo.getProperty(NativeProps.LOGIC_QUEST)).getResult(A, B);
    }

    public void setComplete(UUID uuid, long timestamp) {
        if (uuid == null) {
            return;
        }
        synchronized (this.completeUsers) {
            NBTTagCompound entry = getCompletionInfo(uuid);
            if (entry == null) {
                entry = new NBTTagCompound();
                this.completeUsers.put(uuid, entry);
            }
            entry.func_74757_a("claimed", false);
            entry.func_74772_a("timestamp", timestamp);
        }
    }

    public boolean isComplete(UUID uuid) {
        return ((Boolean) this.qInfo.getProperty(NativeProps.GLOBAL)).booleanValue() ? this.completeUsers.size() > 0 : getCompletionInfo(uuid) != null;
    }

    public EnumQuestState getState(UUID uuid) {
        if (isComplete(uuid)) {
            if (hasClaimed(uuid)) {
                return EnumQuestState.COMPLETED;
            }
            return EnumQuestState.UNCLAIMED;
        }
        if (isUnlocked(uuid)) {
            return EnumQuestState.UNLOCKED;
        }
        return EnumQuestState.LOCKED;
    }

    public NBTTagCompound getCompletionInfo(UUID uuid) {
        NBTTagCompound nBTTagCompound;
        synchronized (this.completeUsers) {
            nBTTagCompound = this.completeUsers.get(uuid);
        }
        return nBTTagCompound;
    }

    public void setCompletionInfo(UUID uuid, NBTTagCompound nbt) {
        if (uuid == null) {
            return;
        }
        synchronized (this.completeUsers) {
            if (nbt == null) {
                this.completeUsers.remove(uuid);
            } else {
                this.completeUsers.put(uuid, nbt);
            }
        }
    }

    public void resetUser(@Nullable UUID uuid, boolean fullReset) {
        synchronized (this.completeUsers) {
            if (fullReset) {
                if (uuid == null) {
                    this.completeUsers.clear();
                } else {
                    this.completeUsers.remove(uuid);
                }
            } else if (uuid == null) {
                this.completeUsers.forEach((key, value) -> {
                    value.func_74757_a("claimed", false);
                    value.func_74772_a("timestamp", 0L);
                });
            } else {
                NBTTagCompound entry = getCompletionInfo(uuid);
                if (entry != null) {
                    entry.func_74757_a("claimed", false);
                    entry.func_74772_a("timestamp", 0L);
                }
            }
            this.tasks.getEntries().forEach(value2 -> {
                ((ITask) value2.getValue()).resetUser(uuid);
            });
        }
    }

    public IDatabaseNBT<ITask, NBTTagList, NBTTagList> getTasks() {
        return this.tasks;
    }

    public IDatabaseNBT<IReward, NBTTagList, NBTTagList> getRewards() {
        return this.rewards;
    }

    @Nonnull
    public int[] getRequirements() {
        return this.preRequisites;
    }

    public void setRequirements(@Nonnull int[] req) {
        this.preRequisites = req;
    }

    public NBTTagCompound writeToNBT(NBTTagCompound jObj) {
        jObj.func_74782_a("properties", this.qInfo.writeToNBT(new NBTTagCompound()));
        jObj.func_74782_a("tasks", this.tasks.writeToNBT(new NBTTagList(), (List) null));
        jObj.func_74782_a("rewards", this.rewards.writeToNBT(new NBTTagList(), (List) null));
        jObj.func_74782_a("preRequisites", new NBTTagIntArray(getRequirements()));
        return jObj;
    }

    public void readFromNBT(NBTTagCompound jObj) {
        this.qInfo.readFromNBT(jObj.func_74775_l("properties"));
        this.tasks.readFromNBT(jObj.func_150295_c("tasks", 10), false);
        this.rewards.readFromNBT(jObj.func_150295_c("rewards", 10), false);
        if (jObj.func_150299_b("preRequisites") == 11) {
            setRequirements(jObj.func_74759_k("preRequisites"));
        } else {
            List<NBTBase> rList = NBTConverter.getTagList(jObj.func_150295_c("preRequisites", 4));
            int[] req = new int[rList.size()];
            for (int i = 0; i < rList.size(); i++) {
                NBTBase nBTBase = rList.get(i);
                req[i] = nBTBase instanceof NBTBase.NBTPrimitive ? ((NBTBase.NBTPrimitive) nBTBase).func_150287_d() : -1;
            }
            setRequirements(req);
        }
        setupProps();
    }

    public NBTTagCompound writeProgressToNBT(NBTTagCompound json, @Nullable List<UUID> users) {
        synchronized (this.completeUsers) {
            NBTTagList comJson = new NBTTagList();
            for (Map.Entry<UUID, NBTTagCompound> entry : this.completeUsers.entrySet()) {
                if (entry.getValue() != null && entry.getKey() != null) {
                    NBTTagCompound tags = entry.getValue().func_74737_b();
                    tags.func_74778_a("uuid", entry.getKey().toString());
                    comJson.func_74742_a(tags);
                }
            }
            json.func_74782_a("completed", comJson);
            NBTTagList tskJson = this.tasks.writeProgressToNBT(new NBTTagList(), users);
            json.func_74782_a("tasks", tskJson);
        }
        return json;
    }

    public void readProgressFromNBT(NBTTagCompound json, boolean merge) {
        synchronized (this.completeUsers) {
            if (!merge) {
                this.completeUsers.clear();
            }
            NBTTagList comList = json.func_150295_c("completed", 10);
            for (int i = 0; i < comList.func_74745_c(); i++) {
                NBTTagCompound entry = comList.func_150305_b(i).func_74737_b();
                try {
                    UUID uuid = UUID.fromString(entry.func_74779_i("uuid"));
                    this.completeUsers.put(uuid, entry);
                } catch (Exception e) {
                    BetterQuesting.logger.log(Level.ERROR, "Unable to load UUID for quest", e);
                }
            }
            this.tasks.readProgressFromNBT(json.func_150295_c("tasks", 10), merge);
        }
    }

    public void setClaimed(UUID uuid, long timestamp) {
        synchronized (this.completeUsers) {
            NBTTagCompound entry = getCompletionInfo(uuid);
            if (entry != null) {
                entry.func_74757_a("claimed", true);
                entry.func_74772_a("timestamp", timestamp);
            } else {
                NBTTagCompound entry2 = new NBTTagCompound();
                entry2.func_74757_a("claimed", true);
                entry2.func_74772_a("timestamp", timestamp);
                this.completeUsers.put(uuid, entry2);
            }
        }
    }

    public <T> T getProperty(IPropertyType<T> iPropertyType) {
        return (T) this.qInfo.getProperty(iPropertyType);
    }

    public <T> T getProperty(IPropertyType<T> iPropertyType, T t) {
        return (T) this.qInfo.getProperty(iPropertyType, t);
    }

    public boolean hasProperty(IPropertyType<?> prop) {
        return this.qInfo.hasProperty(prop);
    }

    public <T> void setProperty(IPropertyType<T> prop, T value) {
        this.qInfo.setProperty(prop, value);
    }

    public void removeProperty(IPropertyType<?> prop) {
        this.qInfo.removeProperty(prop);
    }

    public void removeAllProps() {
        this.qInfo.removeAllProps();
    }
}
